@@ -46,6 +46,7 @@ class Linop():
         oshape: output shape.
         ishape: input shape.
         H: adjoint linear operator.
+        N: normal linear operator.
 
     """
     def __init__(self, oshape, ishape, repr_str=None):
@@ -60,6 +61,9 @@ class Linop():
         else:
             self.repr_str = repr_str
 
+        self.adj = None
+        self.normal = None
+
     def _check_ishape(self, input):
         for i1, i2 in zip(input.shape, self.ishape):
             if i2 != -1 and i1 != i2:
@@ -102,6 +106,9 @@ class Linop():
     def _adjoint_linop(self):
         raise NotImplementedError
 
+    def _normal_linop(self):
+        return self.H * self
+
     @property
     def H(self):
         r"""Return adjoint linear operator.
@@ -116,7 +123,24 @@ class Linop():
             Linop: adjoint linear operator.
 
         """
-        return self._adjoint_linop()
+        if self.adj is None:
+            self.adj = self._adjoint_linop()
+        return self.adj
+
+    @property
+    def N(self):
+        r"""Return normal linear operator.
+
+        A normal linear operator :math:`A^HA` for
+        a linear operator :math:`A`.
+
+        Returns:
+            Linop: adjoint linear operator.
+
+        """
+        if self.normal is None:
+            self.normal = self._normal_linop()
+        return self.normal
 
     def __call__(self, input):
         return self.__mul__(input)
@@ -175,6 +199,9 @@ class Identity(Linop):
     def _adjoint_linop(self):
         return self
 
+    def _normal_linop(self):
+        return self
+
 
 class ToDevice(Linop):
     """Move input between devices.
@@ -652,6 +679,9 @@ class Reshape(Linop):
     def _adjoint_linop(self):
         return Reshape(self.ishape, self.oshape)
 
+    def _normal_linop(self):
+        return Identity(self.ishape)
+
 
 class Transpose(Linop):
     """Tranpose input with the given axes.
@@ -687,6 +717,9 @@ class Transpose(Linop):
 
         return Transpose(oshape, axes=iaxes)
 
+    def _normal_linop(self):
+        return Identity(self.ishape)
+
 
 class FFT(Linop):
     """FFT linear operator.
@@ -712,6 +745,9 @@ class FFT(Linop):
     def _adjoint_linop(self):
         return IFFT(self.ishape, axes=self.axes, center=self.center)
 
+    def _normal_linop(self):
+        return Identity(self.ishape)
+
 
 class IFFT(Linop):
     """IFFT linear operator.
@@ -738,6 +774,9 @@ class IFFT(Linop):
     def _adjoint_linop(self):
         return FFT(self.ishape, axes=self.axes, center=self.center)
 
+    def _normal_linop(self):
+        return Identity(self.ishape)
+
 
 def _get_matmul_oshape(ishape, mshape, adjoint):
     ishape_exp, mshape_exp = util._expand_shapes(ishape, mshape)
@@ -1169,6 +1208,9 @@ class Circshift(Linop):
     def _adjoint_linop(self):
         return Circshift(self.ishape, [-s for s in self.shift], axes=self.axes)
 
+    def _normal_linop(self):
+        return Identity(self.ishape)
+
 
 class Wavelet(Linop):
     """Wavelet transform linear operator.
@@ -1262,7 +1304,6 @@ class Sum(Linop):
             return xp.sum(input, axis=self.axes)
 
     def _adjoint_linop(self):
-
         return Tile(self.ishape, self.axes)
 
 
@@ -1333,6 +1374,9 @@ class ArrayToBlocks(Linop):
     def _adjoint_linop(self):
         return BlocksToArray(self.ishape, self.blk_shape, self.blk_strides)
 
+    def _normal_linop(self):
+        return Identity(self.ishape)
+
 
 class BlocksToArray(Linop):
     """Accumulate blocks into an array in a sliding window manner.
@@ -1365,6 +1409,9 @@ class BlocksToArray(Linop):
     def _adjoint_linop(self):
         return ArrayToBlocks(self.oshape, self.blk_shape, self.blk_strides)
 
+    def _normal_linop(self):
+        return Identity(self.ishape)
+
 
 def Gradient(ishape, axes=None):
     import warnings
@@ -1407,13 +1454,14 @@ class NUFFT(Linop):
         coord (array): Coordinates, with values [-ishape / 2, ishape / 2]
         oversamp (float): Oversampling factor.
         width (float): Kernel width.
-        n (int): Kernel sampling number.
+        toeplitz (bool): Use toeplitz PSF to evaluate normal operator.
 
     """
-    def __init__(self, ishape, coord, oversamp=1.25, width=4):
+    def __init__(self, ishape, coord, oversamp=1.25, width=4, toeplitz=False):
         self.coord = coord
         self.oversamp = oversamp
         self.width = width
+        self.toeplitz = toeplitz
 
         ndim = coord.shape[-1]
 
@@ -1433,6 +1481,23 @@ class NUFFT(Linop):
         return NUFFTAdjoint(self.ishape, self.coord,
                             oversamp=self.oversamp, width=self.width)
 
+    def _normal_linop(self):
+        if self.toeplitz is False:
+            return self.H * self
+
+        ndim = self.coord.shape[-1]
+        psf = fourier.toeplitz_psf(self.coord, self.ishape, self.oversamp,
+                                   self.width)
+
+        fft_axes = tuple(range(-1, -(ndim + 1), -1))
+
+        R = Resize(psf.shape, self.ishape)
+        F = FFT(psf.shape, axes=fft_axes)
+        P = Multiply(psf.shape, psf)
+        T = R.H * F.H * P * F * R
+
+        return T
+
 
 class NUFFTAdjoint(Linop):
     """NUFFT adjoint linear operator.
