@@ -677,14 +677,14 @@ def spokes_grad(k, tbw, sl_thick, gmax, dgdtmax, gts):
         gx.extend([0] * np.size(subgz))  # zeros for gz duration
         if np.absolute(gxarea[ii]) > 0:
             [gblip, _] = trap_grad(abs(gxarea[ii]), gmax, dgdtmax, gts)
-            gxblip = np.int(np.sign(gxarea[ii])) * gblip
+            gxblip = int(np.sign(gxarea[ii])) * gblip
             gx = gx[:len(gx) - len(gxblip.T)]
             gx.extend(np.squeeze(gxblip).tolist())
 
         gy.extend([0] * np.size(subgz))
         if np.absolute(gyarea[ii]) > 0:
             [gblip, _] = trap_grad(abs(gyarea[ii]), gmax, dgdtmax, gts)
-            gyblip = np.int(np.sign(gyarea[ii])) * gblip
+            gyblip = int(np.sign(gyarea[ii])) * gblip
             gy = gy[:len(gy) - len(gyblip.T)]
             gy.extend(np.squeeze(gyblip).tolist())
 
@@ -793,7 +793,7 @@ def runge_kutta(ds: float, st: float, kvals: np.ndarray, smax=None,
 #    (c) Michael Lustig 2005
 #    modified 2006 and 2007
 #    Rewritten in Python in 2020 by Kevin Johnson
-def min_time_gradient(c: np.ndarray, g0=0, gfin=0, gmax=4, smax=15,
+def min_time_gradient(c: np.ndarray, g0=0, gfin=None, gmax=4, smax=15,
                       dt=4e-3, gamma=4.257):
     r"""
     Given a k-space trajectory c(n), gradient and slew constraints. This
